Signed-off-by: Chen Guanqiao <chen.chencha...@foxmail.com>
---
 fs/fat/fat.h                  |  1 +
 fs/fat/inode.c                | 15 ++++++++++++---
 include/uapi/linux/msdos_fs.h |  2 ++
 3 files changed, 15 insertions(+), 3 deletions(-)

diff --git a/fs/fat/fat.h b/fs/fat/fat.h
index 8fc1093da47d..204012c94421 100644
--- a/fs/fat/fat.h
+++ b/fs/fat/fat.h
@@ -86,6 +86,7 @@ struct msdos_sb_info {
        int dir_per_block;            /* dir entries per block */
        int dir_per_block_bits;       /* log2(dir_per_block) */
        unsigned int vol_id;            /*volume ID*/
+       char vol_label[11];             /*volume label*/

        int fatent_shift;
        const struct fatent_operations *fatent_ops;
diff --git a/include/uapi/linux/msdos_fs.h b/include/uapi/linux/msdos_fs.h
index a45d0754102e..90d2d6f3d908 100644
--- a/include/uapi/linux/msdos_fs.h
+++ b/include/uapi/linux/msdos_fs.h
@@ -107,6 +107,8 @@ struct __fat_dirent {
 #define FAT_IOCTL_SET_ATTRIBUTES       _IOW('r', 0x11, __u32)
 /*Android kernel has used 0x12, so we use 0x13*/
 #define FAT_IOCTL_GET_VOLUME_ID                _IOR('r', 0x13, __u32)
+#define FAT_IOCTL_GET_VOLUME_LABEL     _IOR('r', 0x14, __u8[11])
+#define FAT_IOCTL_SET_VOLUME_LABEL     _IOW('r', 0x15, __u8[11])

 struct fat_boot_sector {
        __u8    ignored[3];     /* Boot strap short or near jump */
diff --git a/fs/fat/inode.c b/fs/fat/inode.c
index 20a0a89eaca5..a97b6314ef33 100644
--- a/fs/fat/inode.c
+++ b/fs/fat/inode.c
@@ -45,12 +45,14 @@ struct fat_bios_param_block {

        u8      fat16_state;
        u32     fat16_vol_id;
+       u8      fat16_vol_label[11];

        u32     fat32_length;
        u32     fat32_root_cluster;
        u16     fat32_info_sector;
        u8      fat32_state;
        u32     fat32_vol_id;
+       u8      fat32_vol_label[11];
 };

 static int fat_default_codepage = CONFIG_FAT_DEFAULT_CODEPAGE;
@@ -1460,12 +1462,16 @@ static int fat_read_bpb(struct super_block *sb, struct 
fat_boot_sector *b,

        bpb->fat16_state = b->fat16.state;
        bpb->fat16_vol_id = get_unaligned_le32(b->fat16.vol_id);
+       memcpy(bpb->fat16_vol_label, b->fat16.vol_label,
+                  sizeof(bpb->fat16_vol_label));

        bpb->fat32_length = le32_to_cpu(b->fat32.length);
        bpb->fat32_root_cluster = le32_to_cpu(b->fat32.root_cluster);
        bpb->fat32_info_sector = le16_to_cpu(b->fat32.info_sector);
        bpb->fat32_state = b->fat32.state;
        bpb->fat32_vol_id = get_unaligned_le32(b->fat32.vol_id);
+       memcpy(bpb->fat32_vol_label, b->fat32.vol_label,
+                  sizeof(bpb->fat32_vol_label));

        /* Validate this looks like a FAT filesystem BPB */
        if (!bpb->fat_reserved) {
@@ -1723,11 +1729,14 @@ int fat_fill_super(struct super_block *sb, void *data, 
int silent, int isvfat,
                brelse(fsinfo_bh);
        }

-       /* interpret volume ID as a little endian 32 bit integer */
-       if (sbi->fat_bits == 32)
+       /* interpret volume ID and label as a little endian 32 bit integer */
+       if (sbi->fat_bits == 32) {
                sbi->vol_id = bpb.fat32_vol_id;
-       else /* fat 16 or 12 */
+               memcpy(sbi->vol_label, bpb.fat32_vol_label, 
sizeof(sbi->vol_label));
+       } else { /* fat 16 or 12 */
                sbi->vol_id = bpb.fat16_vol_id;
+               memcpy(sbi->vol_label, bpb.fat16_vol_label, 
sizeof(sbi->vol_label));
+       }

        sbi->dir_per_block = sb->s_blocksize / sizeof(struct msdos_dir_entry);
        sbi->dir_per_block_bits = ffs(sbi->dir_per_block) - 1;
--
2.15.1

Reply via email to